/*
 * Copyright (c) 2012-2017 The Linux Foundation. All rights reserved.
 *
 * Previously licensed under the ISC license by Qualcomm Atheros, Inc.
 *
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * This file was originally distributed by Qualcomm Atheros, Inc.
 * under proprietary terms before Copyright ownership was assigned
 * to the Linux Foundation.
 */

#ifndef __CDS_CONCURRENCY_2X2_DBS_H
#define __CDS_CONCURRENCY_2X2_DBS_H

#include "cds_concurrency.h"

/**
 * second_connection_pcl_dbs_2x2_table - table which provides PCL
 * for the 2nd connection, when we have a connection already in
 * the system (with DBS supported by HW)
 * This table consolidates selection for P2PCLI, P2PGO, STA, SAP
 * into the single set of STA entries for 2.4G and 5G.
 */
static dbs_pcl_second_connection_table_type
second_connection_pcl_dbs_2x2_table = {
	[CDS_STA_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_5G, CDS_5G, CDS_5G} },

	[CDS_STA_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_5G, CDS_5G, CDS_5G} },

	[CDS_STA_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_24G, CDS_24G, CDS_24G} },

	[CDS_STA_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_24G, CDS_24G, CDS_24G} },

	[CDS_P2P_CLI_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_CLI_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_CLI_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_CLI_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_SAP_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_SAP_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_SAP_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },

	[CDS_SAP_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] =	{ CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE} },


	[CDS_IBSS_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] =	{
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_IBSS_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] =	{
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_IBSS_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G, CDS_24G, CDS_24G},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] =	{
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_IBSS_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G, CDS_24G, CDS_24G},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] =	{
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

};

/**
 * third_connection_pcl_dbs_table - table which provides PCL for
 * the 3rd connection, when we have two connections already in
 * the system (with DBS supported by HW)
 */
static dbs_pcl_third_connection_table_type
third_connection_pcl_dbs_2x2_table = {
	[CDS_STA_SAP_SCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_SCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_SCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_SCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G,
				CDS_24G_MCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_DBS_1x1] = {
	[CDS_STA_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24, CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_GO_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_DBS_2x2] = {
	[CDS_STA_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24, CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_GO_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_SAP_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_GO_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_SCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_SCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_SCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_SCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_DBS_1x1] = {
	[CDS_STA_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_DBS_2x2] = {
	[CDS_STA_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_SAP_SAP_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_SCC_24_1x1] = {
		[CDS_STA_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE,
		CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_SCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_SCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_SCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_DBS_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_DBS_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = { CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_GO_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_SCC_24_1x1] = {
	[CDS_STA_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_SCC_24_2x2] = {
	[CDS_STA_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_24_1x1] = {
	[CDS_STA_MODE] = { CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_SCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_SCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_DBS_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_DBS_2x2] = {
	[CDS_STA_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_SAP_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_SCC_24_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_SCC_24_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_MCC_24_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_MCC_24_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_SCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_SCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_MCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_MCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE}, },

	[CDS_STA_P2P_CLI_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_DBS_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_DBS_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_P2P_CLI_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_SCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_SCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_SCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_SCC_CH_24G},
	[CDS_SAP_MODE] = {
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_SCC_CH_24G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_24G_SCC_CH_SBS_CH_5G},
	[CDS_P2P_GO_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_24G_SCC_CH_SBS_CH_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_SCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_SCC_CH_24G},
	[CDS_SAP_MODE] = {
		CDS_24G_SCC_CH_SBS_CH, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_SCC_CH_24G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_24G_SCC_CH_SBS_CH_5G},
	[CDS_P2P_GO_MODE] = {
		CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH_SBS_CH_5G,
		CDS_24G_SCC_CH_SBS_CH_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_MCC_CH, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_MCC_CH, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_24G_MCC_CH, CDS_24G_MCC_CH,
			CDS_24G_MCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_24G_MCC_CH, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_MCC_CH_24G, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {CDS_24G_MCC_CH, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {CDS_MCC_CH_24G, CDS_24G_MCC_CH,
		CDS_24G_MCC_CH},
	[CDS_P2P_GO_MODE] = {CDS_MCC_CH_24G, CDS_24G_MCC_CH, CDS_24G_MCC_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G,		CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_GO_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_DBS_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_GO_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_DBS_2x2] = {
	[CDS_STA_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_GO_MODE] = {
		CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24_5G,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_P2P_GO_P2P_CLI_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_GO_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_SCC_24_1x1] = {
	[CDS_STA_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_SCC_24_2x2] = {
	[CDS_STA_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_SAP_MODE] = {
		CDS_5G_SCC_CH, CDS_5G_SCC_CH, CDS_5G_SCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_MCC_24_1x1] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_MCC_24_2x2] = {
	[CDS_STA_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_SAP_MODE] = {CDS_5G, CDS_5G, CDS_5G},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_SCC_5_1x1] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_SCC_5_2x2] = {
	[CDS_STA_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_SAP_MODE] = {CDS_24G_SCC_CH_SBS_CH_5G, CDS_24G_SCC_CH,
		CDS_24G_SCC_CH_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_MCC_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_MCC_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_SAP_MODE] = {
		CDS_24G_SBS_CH_MCC_CH, CDS_24G, CDS_24G_MCC_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE}, },

	[CDS_STA_STA_MCC_24_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_MCC_24_5_2x2] = {
	[CDS_STA_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_SAP_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_DBS_1x1] = {
	[CDS_STA_MODE] = {CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_DBS_2x2] = {
	[CDS_STA_MODE] = {CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_SAP_MODE] = {CDS_SCC_ON_5_SCC_ON_24_5G, CDS_SCC_ON_5_SCC_ON_24,
		CDS_SCC_ON_5_SCC_ON_24},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

	[CDS_STA_STA_SBS_5_1x1] = {
	[CDS_STA_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_SAP_MODE] = {
		CDS_SBS_CH_5G, CDS_SBS_CH, CDS_SBS_CH},
	[CDS_P2P_CLIENT_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_P2P_GO_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE},
	[CDS_IBSS_MODE] = {
		CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE, CDS_MAX_PCL_TYPE} },

};

/**
 * next_action_two_connection_table - table which provides next
 * action while a new connection is coming up, with one
 * connection already in the system
 */
static next_action_two_connection_table_type
	next_action_two_connection_dbs_2x2_table = {
	[CDS_STA_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_P2P_CLI_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_CLI_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_CLI_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_P2P_CLI_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_P2P_GO_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_P2P_GO_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_P2P_GO_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_P2P_GO_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_SAP_24_1x1] = {CDS_NOP,		CDS_DBS},
	[CDS_SAP_24_2x2] = {CDS_NOP,		CDS_DBS},
	[CDS_SAP_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_SAP_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_IBSS_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_IBSS_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_IBSS_5_1x1] = {CDS_DBS,		CDS_NOP},
	[CDS_IBSS_5_2x2] = {CDS_DBS,		CDS_NOP},
};

/**
 * next_action_three_connection_table - table which provides next
 * action while a new connection is coming up, with two
 * connections already in the system
 */
static next_action_three_connection_table_type
	next_action_three_connection_dbs_2x2_table = {
	[CDS_STA_SAP_SCC_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_SCC_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_MCC_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_MCC_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_SCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_SAP_SCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_SAP_MCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_SAP_MCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_SAP_MCC_24_5_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_MCC_24_5_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_DBS_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_DBS_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_SAP_SBS_5_1x1] = {CDS_DBS_UPGRADE,	CDS_NOP},

	[CDS_STA_P2P_GO_SCC_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_SCC_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_MCC_24_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_MCC_24_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_SCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_P2P_GO_SCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_P2P_GO_MCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_P2P_GO_MCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_P2P_GO_MCC_24_5_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_GO_MCC_24_5_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_GO_DBS_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_DBS_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_GO_SBS_5_1x1] = {CDS_DBS_UPGRADE,	CDS_NOP},

	[CDS_STA_P2P_CLI_SCC_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_SCC_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_MCC_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_MCC_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_SCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_P2P_CLI_SCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_P2P_CLI_MCC_5_1x1] = {CDS_DBS,		CDS_SBS},
	[CDS_STA_P2P_CLI_MCC_5_2x2] = {CDS_DBS,		CDS_SBS_DOWNGRADE},
	[CDS_STA_P2P_CLI_MCC_24_5_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_MCC_24_5_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_P2P_CLI_DBS_1x1] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_CLI_DBS_2x2] = {CDS_NOP,		CDS_NOP},
	[CDS_STA_P2P_CLI_SBS_5_1x1] = {CDS_DBS_UPGRADE, CDS_NOP},

	[CDS_P2P_GO_P2P_CLI_SCC_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_SCC_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_MCC_24_1x1] = {CDS_NOP,	 CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_MCC_24_2x2] = {CDS_NOP,	 CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_SCC_5_1x1] = {CDS_DBS,	  CDS_SBS},
	[CDS_P2P_GO_P2P_CLI_SCC_5_2x2] = {CDS_DBS,	CDS_SBS_DOWNGRADE},
	[CDS_P2P_GO_P2P_CLI_MCC_5_1x1] = {CDS_DBS,	  CDS_SBS},
	[CDS_P2P_GO_P2P_CLI_MCC_5_2x2] = {CDS_DBS,	CDS_SBS_DOWNGRADE},
	[CDS_P2P_GO_P2P_CLI_MCC_24_5_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_MCC_24_5_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_DBS_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_DBS_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_P2P_GO_P2P_CLI_SBS_5_1x1] = {CDS_DBS_UPGRADE,	CDS_NOP},

	[CDS_STA_STA_SCC_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_SCC_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_MCC_24_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_MCC_24_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_SCC_5_1x1] = {CDS_DBS,	CDS_SBS},
	[CDS_STA_STA_SCC_5_2x2] = {CDS_DBS,	CDS_SBS_DOWNGRADE},
	[CDS_STA_STA_MCC_5_1x1] = {CDS_DBS,	CDS_SBS},
	[CDS_STA_STA_MCC_5_2x2] = {CDS_DBS,	CDS_SBS_DOWNGRADE},
	[CDS_STA_STA_MCC_24_5_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_MCC_24_5_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_DBS_1x1] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_DBS_2x2] = {CDS_NOP,	CDS_NOP},
	[CDS_STA_STA_SBS_5_1x1] = {CDS_DBS_UPGRADE, CDS_NOP},

};

#endif
